'use strict';

var _get = require('babel-runtime/helpers/get')['default'];

var _inherits = require('babel-runtime/helpers/inherits')['default'];

var _createClass = require('babel-runtime/helpers/create-class')['default'];

var _classCallCheck = require('babel-runtime/helpers/class-call-check')['default'];

var _extends = require('babel-runtime/helpers/extends')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _mainloopJs = require('mainloop.js');

var _mainloopJs2 = _interopRequireDefault(_mainloopJs);

var FPS = 60;
var TIMESTEP = 1000 / FPS;
var MAX_FPS = 60;

var noop = function noop() {};

/**
 * A wrapper to animate a given component.
 * All props are passed down to the child component.
 *
 * The getUpdate callback takes a reference to the animated component's backing instance,
 * and returns the update function.
 *
 * The update function takes the elapsed time (in milliseconds) since the last update,
 * and returns the props and context for the animated component.
 *
 * @param  {ReactComponent} AnimatedComponent
 * @param  {function} userUpdate
 * @param {function} userBegin
 * @return {ReactComponent} An animated version of the given component.
 */

exports['default'] = function () {
  var timestep = arguments.length <= 0 || arguments[0] === undefined ? TIMESTEP : arguments[0];
  var maxFPS = arguments.length <= 1 || arguments[1] === undefined ? MAX_FPS : arguments[1];
  return function (AnimatedComponent, userUpdate, userBegin) {
    var Animator = (function (_React$Component) {
      _inherits(Animator, _React$Component);

      function Animator(props) {
        _classCallCheck(this, Animator);

        _get(Object.getPrototypeOf(Animator.prototype), 'constructor', this).call(this, props);

        this.state = {
          animatedProps: {}
        };
      }

      _createClass(Animator, [{
        key: 'componentDidMount',
        value: function componentDidMount() {
          var _this = this;

          var begin = userBegin || noop,
              update = function update(delta) {
            var _userUpdate = userUpdate(delta);

            var context = _userUpdate.context;
            var props = _userUpdate.props;

            _this.setState({
              context: context,
              animatedProps: props
            });
          },
              draw = function draw() {
            return (/* interpolationPercentage */_this.forceUpdate()
            );
          },
              endOfFrame = function endOfFrame( /*fps*/_, panic) {
            // TODO let user supply callback for this
            if (panic) {
              loop.resetFrameDelta();
            }
          },
              loop = _mainloopJs2['default'].setMaxAllowedFPS(maxFPS).setSimulationTimestep(timestep).setBegin(begin).setUpdate(update).setDraw(draw).setEnd(endOfFrame);

          this.setState({
            loop: loop,
            animatedProps: {}
          });

          if (this.props.run) {
            loop.start();
          }
        }
      }, {
        key: 'componentWillReceiveProps',
        value: function componentWillReceiveProps(nextProps) {
          if (nextProps.run) {
            this.state.loop.start();
          } else {
            this.state.loop.stop();
          }
        }
      }, {
        key: 'shouldComponentUpdate',
        value: function shouldComponentUpdate() {
          // doesn't affect initial render
          return false; // take control of when we render using forceUpdate
        }
      }, {
        key: 'componentWillUnmount',
        value: function componentWillUnmount() {
          this.state.loop.stop();
        }
      }, {
        key: 'getChildContext',
        value: function getChildContext() {
          return {
            animContext: this.state.context
          };
        }
      }, {
        key: 'render',
        value: function render() {
          return _react2['default'].createElement(AnimatedComponent, _extends({ ref: 'animated'
          }, this.props, this.state.animatedProps, {
            context: this.state.context
          }));
        }
      }]);

      return Animator;
    })(_react2['default'].Component);

    Animator.propTypes = {
      run: _react2['default'].PropTypes.bool
    };

    Animator.defaultProps = {
      run: true
    };

    Animator.childContextTypes = {
      animContext: _react2['default'].PropTypes.any
    };

    return Animator;
  };
};

module.exports = exports['default'];